﻿using System;
using Windows.UI.Xaml;
using Windows.UI.Xaml.Controls;

//追加
using System.Text;
using System.Diagnostics;
using System.Threading.Tasks;
using Windows.Devices.Enumeration;
using Windows.Devices.I2c;

// 空白ページのアイテム テンプレートについては、http://go.microsoft.com/fwlink/?LinkId=402352&clcid=0x409 を参照してください

namespace Step4_1
{
    /// <summary>
    /// それ自体で使用できる空白ページまたはフレーム内に移動できる空白ページ。
    /// </summary>
    public sealed partial class MainPage : Page
    {
        //LCD
        private I2cDevice LCD;
        private byte LCD_Addr = 0x3e;

        //BME280
        private I2cDevice sensor;
        private byte BME280_Addr = 0x76;

        //センサーデータ取得用タイマー
        private DispatcherTimer timer;

        //データ取得用変数
        private double TMP;
        private double PRE;
        private double HUM;

        //BME280 キャリブレーションデータ用変数
        private UInt16 T1;
        private Int16 T2;
        private Int16 T3;
        private UInt16 P1;
        private Int16 P2;
        private Int16 P3;
        private Int16 P4;
        private Int16 P5;
        private Int16 P6;
        private Int16 P7;
        private Int16 P8;
        private Int16 P9;
        private byte H1;
        private Int16 H2;
        private byte H3;
        private Int16 H4;
        private Int16 H5;
        private Int16 H6;

        //BME280 データレジスタアドレス
        private byte HumLsbAddr = 0xfe;
        private byte HumMsbAddr = 0xfd;
        private byte TmpXlsbAddr = 0xfc;
        private byte TmpLsbAddr = 0xfb;
        private byte TmpMsbAddr = 0xfa;
        private byte PreXlsbAddr = 0xf9;
        private byte PreLsbAddr = 0xf8;
        private byte PreMsbAddr = 0xf7;

        //BME280　データ校正用変数
        private Int32 t_fine= Int32.MinValue;


        //BME280 キャリブレーションデータアドレス
        enum Register : byte
        {
            dig_T1 = 0x88,
            dig_T2 = 0x8a,
            dig_T3 = 0x8c,

            dig_P1 = 0x8e,
            dig_P2 = 0x90,
            dig_P3 = 0x92,
            dig_P4 = 0x94,
            dig_P5 = 0x96,
            dig_P6 = 0x98,
            dig_P7 = 0x9a,
            dig_P8 = 0x9c,
            dig_P9 = 0x9e,

            dig_H1 = 0xa1,
            dig_H2 = 0xe1,
            dig_H3 = 0xe3,
            dig_H4 = 0xe4,
            dig_H5 = 0xe5,
            dig_H6 = 0xe7,
        }

        //BME280 測定データ取得レジスタアドレス
        enum Cmd : byte
        {
            READ_TMP = 0xfa,
            READ_PRE = 0xf7,
            READ_HUM = 0xfd,
        }

        /// <summary>
        /// 
        /// </summary>
        public MainPage()
        {
            this.InitializeComponent();

            //デバイスの初期化
            InitDevice();

            //タイマーのセットアップ
            timer = new DispatcherTimer();
            timer.Interval = TimeSpan.FromSeconds(10);
            timer.Tick += Timer_Tick;
            timer.Start();
            
        }

        private async void Timer_Tick(object sender, object e)
        {
            //センサーデータ取得

            TMP = await readTmp();
            PRE = await readPre();
            HUM = await readHum();

            //LCDに表示
            await WriteCmd(0x80, 1);
            await WriteDisp("T:" + TMP.ToString("F1") + " H:" + HUM.ToString("F1"));    //1行目表示
            await WriteCmd(0xc0, 1);                                                    //2行目にカーソル移動
            await WriteDisp("P:" + PRE.ToString("F1"));                                 //2行目表示

            //デバッグ用
            Debug.WriteLine("TMP: " + TMP.ToString("F1"));
            Debug.WriteLine("PRE: " + PRE.ToString("F1"));
            Debug.WriteLine("HUM: " + HUM.ToString("F1"));
        }

        /// <summary>
        /// デバイスの初期化
        /// </summary>
        private async void InitDevice()
        {
            //I2C コントローラーの取得（I2Cデバイスが複数あってもコントローラーは共通。RaspberryPi2はI2Cコントローラーは1つ）
            var aqs = I2cDevice.GetDeviceSelector("I2C1");
            var dis = await DeviceInformation.FindAllAsync(aqs);

            //BME280初期化
            var BME280Setting = new I2cConnectionSettings(BME280_Addr);
            BME280Setting.BusSpeed = I2cBusSpeed.FastMode;
            sensor = await I2cDevice.FromIdAsync(dis[0].Id, BME280Setting);
            await sensorConfig();
            calibration();


            //LCDの初期化
            var LcdSetting = new I2cConnectionSettings(LCD_Addr);
            LcdSetting.BusSpeed = I2cBusSpeed.FastMode;
            LCD = await I2cDevice.FromIdAsync(dis[0].Id, LcdSetting);
            await lcdConfig();
        }

        /// <summary>
        /// BME280のコンフィグレーション
        /// </summary>
        private async Task sensorConfig()
        {
            
            uint osrs_t = 3;            //温度
            uint osrs_p = 3;
            uint osrs_h = 3;
            uint mode = 3;
            uint t_sb = 5;
            uint filter = 0;
            uint spi3w_en = 0;

            uint ctrlMeasReg = (osrs_t << 5) | (osrs_p << 2) | mode;
            uint configReg = (t_sb << 5) | (filter << 2) | spi3w_en;
            uint ctrlHumReg = osrs_h;

            sensor.Write(new byte[] { 0xf2, (byte)ctrlHumReg });
            sensor.Write(new byte[] { 0xf4, (byte)ctrlMeasReg });
            sensor.Write(new byte[] { 0xf5, (byte)configReg });

            await Task.Delay(10);

        }

        /// <summary>
        /// BME280　キャリブレーションデータの読み込み
        /// </summary>
        private void calibration()
        {

            //温度
            T1 = ReadUInt16((byte)Register.dig_T1);
            T2 = (Int16)ReadUInt16((byte)Register.dig_T2);
            T3 = (Int16)ReadUInt16((byte)Register.dig_T3);

            //気圧
            P1 = ReadUInt16((byte)Register.dig_P1);
            P2 = (Int16)ReadUInt16((byte)Register.dig_P2);
            P3 = (Int16)ReadUInt16((byte)Register.dig_P3);
            P4 = (Int16)ReadUInt16((byte)Register.dig_P4);
            P5 = (Int16)ReadUInt16((byte)Register.dig_P5);
            P6 = (Int16)ReadUInt16((byte)Register.dig_P6);
            P7 = (Int16)ReadUInt16((byte)Register.dig_P7);
            P8 = (Int16)ReadUInt16((byte)Register.dig_P8);
            P9 = (Int16)ReadUInt16((byte)Register.dig_P9);

            //湿度
            H1 = ReadByte((byte)Register.dig_H1);
            H2 = (Int16)ReadUInt16((byte)Register.dig_H2);
            H3 = ReadByte((byte)Register.dig_H3);
            H4 = (short)(ReadByte((byte)Register.dig_H4) << 4 | ReadByte((byte)Register.dig_H4 + 1) & 0xf);
            H5 = (short)(ReadByte((byte)Register.dig_H5 + 1) << 4 | ReadByte((byte)Register.dig_H5) >> 4);
            H6 = (sbyte)ReadByte((byte)Register.dig_H6);

        }

        /// <summary>
        /// 2バイトデータ読み出し
        /// </summary>
        /// <param name="register"></param>
        /// <returns></returns>
        private UInt16 ReadUInt16(byte register)
        {
            byte[] writeBuf = new byte[] { 0x00 };
            byte[] readBuf = new byte[] { 0x00, 0x00 };

            writeBuf[0] = register;
            sensor.WriteRead(writeBuf, readBuf);

            int h = readBuf[1] << 8;
            int l = readBuf[0];

            return (UInt16)(h + l);
        }

        /// <summary>
        /// バイトデータ読み出し
        /// </summary>
        /// <param name="register"></param>
        /// <returns></returns>
        private byte ReadByte(byte register)
        {
            byte[] writeBuf = new byte[] { 0x00 };
            byte[] readBuf = new byte[] { 0x00 };

            writeBuf[0] = register;
            sensor.WriteRead(writeBuf, readBuf);

            return readBuf[0];
        }

        /// <summary>
        /// 温度データ取得
        /// </summary>
        /// <returns></returns>
        private async Task<double> readTmp()
        {
            byte tmsb = ReadByte(TmpMsbAddr);
            byte tlsb = ReadByte(TmpLsbAddr);
            byte txlsb = ReadByte(TmpXlsbAddr);

            Int32 tmpRaw = (tmsb << 12) | (tlsb << 4) | (txlsb >> 4);

            double var1, var2, T;

            var1 = ((tmpRaw / 16384.0) - (T1 / 1024.0)) * T2;
            var2 = ((tmpRaw / 131072.0) - (T1 / 8192.0)) * T3;
            t_fine = (Int32)(var1 + var2);

            T = (var1 + var2) / 5120.0;

            await Task.Delay(1);

            return T;     
        }

        /// <summary>
        /// 気圧データ取得
        /// </summary>
        /// <returns></returns>
        private async Task<double> readPre()
        {
            /*
            if (t_fine == Int32.MinValue)
            {
                TMP = await readTmp();
            }
            */

            byte pmsb = ReadByte(PreMsbAddr);
            byte plsb = ReadByte(PreLsbAddr);
            byte pxlsb = ReadByte(PreXlsbAddr);

            Int32 preRaw = (pmsb << 12) | (plsb << 4) | (pxlsb >> 4);

            Int64 var1, var2, P;

            var1 = t_fine - 128000;
            var2 = var1 * var1 * (Int64)P6;
            var2 = var2 + ((var1 * (Int64)P5) << 17);
            var2 = var2 + ((Int64)P4 << 35);
            var1 = ((var1 * var1 * (Int64)P3) >> 8) + ((var1 * (Int64)P2) << 12);
            var1 = (((((Int64)1 << 47) + var1)) * (Int64)P1) >> 33;
            if (var1 == 0)
            {
                return 0;
            }

            P = 1048576 - preRaw;
            P = (((P << 31) - var2) * 3125) / var1;
            var1 = ((Int64)P9 * (P >> 13)) >> 25;
            var2 = ((Int64)P8 * P) >> 19;
            P = ((P + var1 + var2) >> 8) + ((Int64)P7 << 4);

            await Task.Delay(1);

            return (double)(P/256/100);
        }

        /// <summary>
        /// 湿度データ取得
        /// </summary>
        /// <returns></returns>
        private async Task<double> readHum()
        {
            byte hmsb = ReadByte(HumMsbAddr);
            byte hlsb = ReadByte(HumLsbAddr);
            int humRaw = (hmsb << 8) | hlsb;

            Int32 H;
            H = t_fine - 76800;
            H = (((((humRaw << 14) - (((Int32)H4) << 20) - ((Int32)H5 * H)) + ((Int32)16384)) >> 15) * (((((((H * ((Int32)H6)) >> 10) * (((H * ((Int32)H3)) >> 11) + ((Int32)32768))) >> 10) + ((Int32)2097152)) * ((Int32)H2) + 8192) >> 14));
            H = (H - (((((H >> 15) * (H >> 15)) >> 7) * ((Int32)H1)) >> 4));
            H = (H < 0 ? 0 : H);
            H = (H > 419430400 ? 419430400 : H);

            await Task.Delay(1);

            return (UInt32)((H>>12)/1000);
        }

        /// <summary>
        /// LCDの設定
        /// </summary>
        private async Task lcdConfig()
        {
            await Task.Delay(50);

            await WriteCmd(0x38, 1);            //行数設定
            await WriteCmd(0x39, 1);            //拡張コマンド設定
            await WriteCmd(0x14, 1);            //内部OSC周波数設定
            await WriteCmd(0x71, 1);            //コントラスト設定
            await WriteCmd(0x56, 1);            //パワー・コントラスト設定
            await WriteCmd(0x6c, 250);          //Follower設定
            await WriteCmd(0x38, 1);            //拡張コマンド設定終了
            await WriteCmd(0x0c, 1);            //ディスプレイOn
            await WriteCmd(0x01, 200);          //ディスプレイクリア
        }

        /// <summary>
        /// LCD表示用メソッド
        /// </summary>
        /// <param name="msg"></param>
        /// <returns></returns>
        private async Task WriteDisp(string msg)
        {
            byte[] bytemsg = Encoding.ASCII.GetBytes(msg);

            for (int i = 0; i <= msg.Length - 2; i++)
            {
                LCD.Write(new byte[] { 0x40, bytemsg[i] });
            }

            LCD.Write(new byte[] { 0x40, bytemsg[msg.Length - 1] });

            await Task.Delay(1);
        }

        /// <summary>
        /// LCD設定用メソッド
        /// </summary>
        /// <param name="Cmd"></param>
        /// <param name="time"></param>
        /// <returns></returns>
        private async Task WriteCmd(byte Cmd, int time)
        {
            LCD.Write(new byte[] { 0, Cmd });
            await Task.Delay(time);
        }

    }
}
