﻿using System;
using Windows.UI.Xaml;
using Windows.UI.Xaml.Controls;

//追加
using Windows.Media.Capture;                    //動画を撮影用
using Windows.Media.MediaProperties;            //動画のプロパティを設定用
using System.Threading;                         //タイマー用
using System.Threading.Tasks;                   //タイマー用
using Windows.Storage;                          //動画ファイル用
using Windows.Devices.Gpio;                     //GPIO用
using Windows.Devices.Enumeration;          
using System.Diagnostics;                       //デバッグ用
using Microsoft.WindowsAzure.Storage;           //Azure Storage用
using Microsoft.WindowsAzure.Storage.Auth;      //Azure Storage用
using Microsoft.WindowsAzure.Storage.Blob;      //Azure Storage用

// 空白ページのアイテム テンプレートについては、http://go.microsoft.com/fwlink/?LinkId=402352&clcid=0x409 を参照してください

namespace Step4_2
{
    /// <summary>
    /// それ自体で使用できる空白ページまたはフレーム内に移動できる空白ページ。
    /// </summary>
    public sealed partial class MainPage : Page
    {
        private MediaCapture capture;                   //キャプチャーオブジェクト
        private StorageFile videoFile;                  //動画保存用
        private const string fileName = "video.mp4";    //保存用ファイル名
        private const string storageName = "[StorageName]";  //アップロード先のストレージ名
        private const string storageKey = "[StorageKey]";   //アップロード先のアクセスKey

        private Timer timer;                //録画用タイマー
        private bool isRecording=false;     //録画用フラグ
        private bool isSensor=false;        //センサー用フラグ

        private const int inputPin = 5;     //センサー入力用GPIOピン
        private const int actPin = 47;      //動作確認用ACTLED
        private GpioPin pin;                
        private GpioPin statusPin;

        /// <summary>
        /// 
        /// </summary>
        public MainPage()
        {
            this.InitializeComponent();
            
            Loaded += MainPage_Loaded;
        }

        private async void MainPage_Loaded(object sender, RoutedEventArgs e)
        {
            await initWebcam();
            await initGpio();
        }
        
        /// <summary>
        /// 
        /// </summary>
        /// <returns></returns>
        private async Task initWebcam()
        {
            Debug.WriteLine("Webカメラ初期化中・・・");

            try
            {
                if (capture != null)
                {
                    if (isRecording)
                    {
                        await capture.StopRecordAsync();
                        isRecording = false;
                    }

                    capture.Dispose();
                    capture = null;
                }

                var devices = await DeviceInformation.FindAllAsync(DeviceClass.VideoCapture);

                if (devices.Count == 0)
                {
                    Debug.WriteLine("Webカメラが見つかりません。");
                    return;
                }

                capture = new MediaCapture();
                await capture.InitializeAsync();

                await Task.Delay(10000);

                Debug.WriteLine("Webカメラ初期化完了");

            }
            catch(Exception ex)
            {
                Debug.WriteLine(ex.Message);
            }
            
        }

        /// <summary>
        /// 
        /// </summary>
        private async Task initGpio()
        {
            var gpio = GpioController.GetDefault();
            if (gpio == null)
            {
                Debug.WriteLine("GPIOコントローラーが見つかりません。");
                return;
            }

            //GPIO入力の設定
            pin = gpio.OpenPin(inputPin);

            if (pin.IsDriveModeSupported(GpioPinDriveMode.InputPullUp))
            {
                pin.SetDriveMode(GpioPinDriveMode.InputPullUp);
            }
            else
            {
                pin.SetDriveMode(GpioPinDriveMode.Input);
            }

            //ACT LEDの設定
            statusPin = gpio.OpenPin(actPin);
            statusPin.Write(GpioPinValue.Low);
            statusPin.SetDriveMode(GpioPinDriveMode.Output);

            Debug.WriteLine("GPIO初期化中・・・");

            //wait
            await Task.Delay(10000);

            //イベント登録
            pin.ValueChanged += Pin_ValueChanged;

            Debug.WriteLine("GPIO初期化完了");
            statusPin.Write(GpioPinValue.High);
        }

        /// <summary>
        /// 
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="args"></param>
        private void Pin_ValueChanged(GpioPin sender, GpioPinValueChangedEventArgs args)
        {  
            if (args.Edge == GpioPinEdge.RisingEdge)
            {
                Debug.WriteLine("Rise");
                if (isRecording == false && isSensor==false)
                {     
                    startRecording();               
                }
            }
            if (args.Edge == GpioPinEdge.FallingEdge)
            {
                Debug.WriteLine("Fall");
                
                if (isSensor == true)
                {
                    isSensor = false;
                }
            }
        }

        /// <summary>
        /// 
        /// </summary>
        private async void startRecording()
        {
            if (isSensor == false)
            {
                //Setup Recording File
                videoFile = await KnownFolders.VideosLibrary.CreateFileAsync(fileName, CreationCollisionOption.ReplaceExisting);

                MediaEncodingProfile recordProfile = null;
                recordProfile = MediaEncodingProfile.CreateMp4(VideoEncodingQuality.Qvga);

                await capture.StartRecordToStorageFileAsync(recordProfile, videoFile);

                isRecording = true;
                isSensor = true;
                Debug.WriteLine("Start Recording");

                timer = new Timer(TimerCallback, null, 15000, 15000);
            }
        }

        /// <summary>
        /// 
        /// </summary>
        /// <param name="state"></param>
        private async void TimerCallback(object state)
        {
            if (isRecording == true)
            {
                timer.Dispose();
                await capture.StopRecordAsync();
                
                Debug.WriteLine("Stop Recording");

                await fileUpload();

                isRecording = false;

                Debug.WriteLine("アップロード完了");
            }   
        }

        /// <summary>
        /// 
        /// </summary>
        /// <returns></returns>
        private async Task fileUpload()
        {
            Debug.WriteLine("アップロード開始");

            //AzureStorageの接続設定（HTTPS接続）
            StorageCredentials sc = new StorageCredentials(storageName, storageKey);
            CloudStorageAccount storageAccount = new CloudStorageAccount(sc, true);

            //コンテナ設定
            CloudBlobClient blobClient = storageAccount.CreateCloudBlobClient();
            CloudBlobContainer container = blobClient.GetContainerReference("video");
            await container.CreateIfNotExistsAsync();

            //Blobへアップロード
            string uploadFilename = "video" + DateTime.UtcNow.ToString("yyyyMMddmmss") + ".mp4";
            CloudBlockBlob blockBlob = container.GetBlockBlobReference(uploadFilename);
            StorageFile storageFile = await KnownFolders.VideosLibrary.GetFileAsync("video.mp4");
            await blockBlob.UploadFromFileAsync(storageFile);     
        }
    }
}
